<?php

namespace Drupal\xtcfield\Plugin\Field\FieldType;

use Drupal\Component\Utility\Random;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemBase;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\Core\TypedData\DataDefinition;
use Drupal\xtc\XtendedContent\API\ToolBox;

/**
 * Plugin implementation of the 'xtcfield_profile' field type.
 *
 * @FieldType(
 *   id = "xtcfield_profile",
 *   label = @Translation("XTC Profile Field"),
 *   description = @Translation("Access data through an XTC Profile"),
 *   category = @Translation("XTC Fields"),
 *   default_widget = "xtcfield_options_select",
 *   default_formatter = "xtcfield_label_formatter",
 *   list_class = "\Drupal\Core\Field\FieldItemList",
 * )
 */
class XtcFieldProfile extends FieldItemBase {

  /**
   * {@inheritdoc}
   */
  public static function defaultStorageSettings() {
    return [
        'max_length' => 255,
        'xtcprofile' => '',      //Default XTC Profile
        'xtcoptions' => '',      //Default Options for XTC Profile
        'xtcprofile_item' => '',      //Default XTC Profile - Item
        'xtcoptions_item' => '',      //Default Options for XTC Profile - Item
        'count' => 10,           //Default default number of results
        'is_ascii' => FALSE,
        'case_sensitive' => FALSE,
      ] + parent::defaultStorageSettings();
  }

  /**
   * {@inheritdoc}
   */
  public static function propertyDefinitions(FieldStorageDefinitionInterface $field_definition) {
    // Prevent early t() calls by using the TranslatableMarkup.
    $properties['value'] = DataDefinition::create('string')
      ->setLabel(new TranslatableMarkup('Text value'))
      ->setSetting('case_sensitive', $field_definition->getSetting('case_sensitive'))
      ->setRequired(TRUE);

    return $properties;
  }

  /**
   * {@inheritdoc}
   */
  public static function schema(FieldStorageDefinitionInterface $field_definition) {
    return [
      'columns' => [
        'value' => [
          'type' => 'varchar',
          'length' => (int) $field_definition->getSetting('max_length'),
          'binary' => $field_definition->getSetting('case_sensitive'),
        ],
      ],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getConstraints() {
    $constraints = parent::getConstraints();

    if ($max_length = $this->getSetting('max_length')) {
      $constraint_manager = \Drupal::typedDataManager()->getValidationConstraintManager();
      $constraints[] = $constraint_manager->create('ComplexData', [
        'value' => [
          'Length' => [
            'max' => $max_length,
            'maxMessage' => t('%name: may not be longer than @max characters.', [
              '%name' => $this->getFieldDefinition()->getLabel(),
              '@max' => $max_length
            ]),
          ],
        ],
      ]);
    }

    return $constraints;
  }

  /**
   * {@inheritdoc}
   */
  public static function generateSampleValue(FieldDefinitionInterface $field_definition) {
    $random = new Random();
    $values['value'] = $random->word(mt_rand(1, $field_definition->getSetting('max_length')));
    return $values;
  }

  /**
   * {@inheritdoc}
   */
  public function storageSettingsForm(array &$form, FormStateInterface $form_state, $has_data) {
    $has_data = FALSE;
    $elements = [];
    $elements['xtcprofile'] = [
      '#type' => 'select',
      '#title' => t('XTC Profile - List'),
      '#default_value' => $this->getSetting('xtcprofile'),
      '#description' => t('Xtended Content options for profile'),
      '#options' => ToolBox::getPlugins(),
      '#required' => TRUE,
    ];
    $elements['xtcoptions'] = [
      '#type' => 'textarea',
      '#title' => t('XTC Profile options - List'),
      '#default_value' => $this->getSetting('xtcoptions'),
      '#description' => t('Xtended Content data profile. Define line by line in a "key|value" way.'),
      '#required' => FALSE,
    ];
    $elements['xtcprofile_item'] = [
      '#type' => 'select',
      '#title' => t('XTC Profile - Item'),
      '#default_value' => $this->getSetting('xtcprofile_item'),
      '#description' => t('Xtended Content options for profile'),
      '#options' => ToolBox::getPlugins(),
      '#required' => TRUE,
    ];
    $elements['xtcoptions_item'] = [
      '#type' => 'textarea',
      '#title' => t('XTC Profile options - Item'),
      '#default_value' => $this->getSetting('xtcoptions_item'),
      '#description' => t('Xtended Content data profile. Define line by line in a "key|value" way.'),
      '#required' => FALSE,
    ];
    $elements['count'] = [
      '#type' => 'number',
      '#title' => t('Max result count'),
      '#default_value' => $this->getSetting('count') ?? 10,
      '#required' => TRUE,
      '#min' => 1,
    ];
    $elements['max_length'] = [
      '#type' => 'number',
      '#title' => t('Maximum length'),
      '#default_value' => $this->getSetting('max_length'),
      '#required' => TRUE,
      '#description' => t('The maximum length of the field in characters.'),
      '#min' => 1,
    ];

    return $elements;
  }

}
